-- ************************************************************************** --
-- * TitanSpec.lua                                                         
-- *                                                                        
-- * Original Author: Rothpada   
-- * Maintainer: Nasapunk88                                                       
-- ************************************************************************** --

local TITAN_SPEC_ID = "TitanSpec";
local L = LibStub("AceLocale-3.0"):GetLocale("TitanSpec", true)
local AceTimer = LibStub("AceTimer-3.0"); 
local IS_DEBUGGING = ( UnitName("player") == "Balther" and GetRealmName() == "Shadow Council" )
						or ( UnitName("player") == "Rothpada" and GetRealmName() == "Broxigar" )
						or ( UnitName("player") == "Furrocious" and GetRealmName() == "Shadowsong" );

-- ************************************************************************** --
-- ******************************** Spec Change Chat Filter ************************* --
-- ************************************************************************** --
local f = CreateFrame"Frame" 

local orgSetActiveSpecGroup = _G.SetActiveSpecGroup 

local spam1 = gsub(ERR_LEARN_ABILITY_S:gsub("%.", "%."), "%%s", "(.*)") 
local spam2 = gsub(ERR_LEARN_SPELL_S:gsub("%.", "%."), "%%s", "(.*)") 
local spam3 = gsub(ERR_SPELL_UNLEARNED_S:gsub("%.", "%."), "%%s", "(.*)") 
local spam4 = gsub(ERR_LEARN_PASSIVE_S:gsub("%.", "%."), "%%s", "(.*)") 

local function SpamFilter(self, event, msg) 
    if strfind(msg, spam1) or strfind(msg, spam2) or strfind(msg, spam3) or strfind(msg, spam4) then return true end 
end 

function SetActiveSpecGroup(...) 
    f:RegisterEvent"UNIT_SPELLCAST_SUCCEEDED" 
    f:RegisterEvent"UNIT_SPELLCAST_STOP" 
    ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", SpamFilter) 
    return orgSetActiveSpecGroup(...) 
end 

f:SetScript("OnEvent", function(self, event, unit) 
    if unit ~= "player" then return end 
    self:UnregisterEvent"UNIT_SPELLCAST_SUCCEEDED" 
    self:UnregisterEvent"UNIT_SPELLCAST_STOP" 
    ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", SpamFilter) 
end)  															
								
-- ************************************************************************** --
-- ******************************** Titan Functions ************************* --
-- ************************************************************************** --

function TitanPanelTitanSpecButton_OnLoad(self)
	self.registry = {
		id = TITAN_SPEC_ID,
		menuText = L["TITAN_SPEC_MENU_TEXT"],
		buttonTextFunction = "TitanPanelTitanSpecButton_GetButtonText",
		tooltipTitle = L["TITAN_SPEC_TOOLTIP_TITLE"],
		tooltipTextFunction = "TitanPanelTitanSpecButton_GetTooltipText",
		savedVariables = {
			ShowLabelText = 1,
			ShowColours = 1,
			ShowGearSet = 1,
			CurrentGearSet = 0,
		}	
	};
	self:RegisterEvent("ADDON_LOADED");
    --self:RegisterEvent("PLAYER_LOGOUT");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("CHARACTER_POINTS_CHANGED");
	self:RegisterEvent("PLAYER_TALENT_UPDATE");
	self:RegisterEvent("CHAT_MSG_SYSTEM");
	
	TitanSpec_output(TITAN_SPEC_ID.." by Rothpada (Shadow Council)");
	TitanSpec_refresh();
	TitanPanelPluginHandle_OnUpdate({TITAN_SPEC_ID, TITAN_PANEL_UPDATE_ALL});
	AceTimer.ScheduleTimer(TITAN_SPEC_ID, function() 
            TitanSpec_refresh();
            TitanPanelPluginHandle_OnUpdate({TITAN_SPEC_ID, TITAN_PANEL_UPDATE_ALL});
        end, 10);
end

function TitanPanelTitanSpecButton_OnClick(self, button)
	if ( button == "LeftButton" ) then
		if ( GetNumSpecGroups() > 1 ) then
				local idx = mod(GetActiveSpecGroup(), GetNumSpecGroups()) + 1;
				if ( IsShiftKeyDown() ) then
						SetActiveSpecGroup(idx);
				elseif ( IsAltKeyDown() ) then
						local eset = TitanSpec_EquipmentSet;
						local cset = TitanGetVar(TITAN_SPEC_ID, "CurrentGearSet");
						if (cset == 0) then
								if ( eset ~= nil and eset[GetActiveSpecGroup()] ~= nil and strtrim(eset[GetActiveSpecGroup()]) ~= "" ) then
										UseEquipmentSet(eset[GetActiveSpecGroup()]);
										TitanSetVar(TITAN_SPEC_ID, "CurrentGearSet", GetActiveSpecGroup());
								end
						else
							local oset = mod(cset, GetNumSpecGroups()) + 1;
							if ( eset ~= nil and eset[oset] ~= nil and strtrim(eset[oset]) ~= "" ) then
									UseEquipmentSet(eset[oset]);
									TitanSetVar(TITAN_SPEC_ID, "CurrentGearSet", oset);
							end
						end
				else
						SetActiveSpecGroup(idx);
						AceTimer.ScheduleTimer(TITAN_SPEC_ID, function()
							if(GetActiveSpecGroup() == idx) then
								-- Ticket 5
								local eset = TitanSpec_EquipmentSet;
								if ( eset ~= nil and eset[idx] ~= nil and strtrim(eset[idx]) ~= "" ) then
										UseEquipmentSet(eset[idx]);
										TitanSetVar(TITAN_SPEC_ID, "CurrentGearSet", idx);
										TitanPanelButton_UpdateButton(TITAN_SPEC_ID);
								end
							end
						end, 6);
				end
		end
		TitanPanelPluginHandle_OnUpdate({TITAN_SPEC_ID, TITAN_PANEL_UPDATE_ALL});
	end
end

function TitanPanelTitanSpecButton_OnEvent(self, event, ...)
	if event == "ADDON_LOADED" and ... == "TitanSpec" then
		TitanSpec_EquipmentSet = TitanSpec_EquipmentSet or {"","",};
		TitanSpec_CustomLabel = TitanSpec_CustomLabel or {"","",};
		TitanSpec_BuildName = TitanSpec_BuildName or {"","",};
		self:UnregisterEvent("ADDON_LOADED")
    end    
   -- if event == "PLAYER_LOGOUT" then
   --	TitanSpec_EquipmentSet = TitanSpec_EquipmentSet;
   --end
	TitanSpec_refresh();
	TitanPanelButton_UpdateButton(TITAN_SPEC_ID);
	TitanPanelPluginHandle_OnUpdate({TITAN_SPEC_ID, TITAN_PANEL_UPDATE_ALL});
end

function TitanSpec_refresh()	
	--TitanSpec_debug("Refreshing...");
	local buildName = TitanSpec_BuildName;
	local curBuild = TitanSpec_getCurrentBuildName();
	if ( buildName ~= nil and curBuild ~= nil) then
		buildName[GetActiveSpecGroup()] = curBuild;
		TitanSpec_BuildName = buildName;
	end
end

function TitanSpec_getCurrentBuildName() 		
	local g = GetSpecialization();
	if ( g ~= nil ) then
		local _,specname,_,_,_,_ = GetSpecializationInfo(g);
		return specname;
	else 
		return g;
	end
end

function TitanPanelTitanSpecButton_GetButtonText(id)
	local specNum = GetActiveSpecGroup();
	local label = L["TITAN_SPEC_SPEC"].." "..specNum..": ";

	-- Build the button text	
	local customLabel = TitanSpec_CustomLabel;
	-- Fix for ticket 7
	if ( customLabel[specNum] == nil or type(customLabel[specNum]) == "boolean" ) then
	        customLabel[specNum] = "";
	        TitanSpec_CustomLabel = customLabel;
	end

	local buildText;
	if ( customLabel == nil or strtrim(customLabel[specNum]) == "" ) then
		buildText = TitanSpec_getCurrentBuildName();
	else
		buildText = customLabel[specNum];
	end
	
	local ret = buildText;
	if ( TitanGetVar(TITAN_SPEC_ID, "ShowGearSet") ) then
                local sets = TitanSpec_EquipmentSet;
				local cset = TitanGetVar(TITAN_SPEC_ID, "CurrentGearSet");
				local set = sets[cset];
                if ( set ~= nil and set ~= "" ) then
	                ret = format("%s [%s]", ret, set);
                end
	end
	return label, TitanSpec_getColouredText(ret, "white");
end

function TitanPanelTitanSpecButton_GetTooltipText(self)
	local CustomLabel = TitanSpec_CustomLabel;
	local BuildName = TitanSpec_BuildName;
	local EquipmentSet = TitanSpec_EquipmentSet;
	
	local t1 = L["TITAN_SPEC_PRIMARY_SPEC"];
	local t2 = "\r"..L["TITAN_SPEC_SECONDARY_SPEC"];
	
	if ( CustomLabel[1] ~= nil and strtrim(CustomLabel[1]) ~= "" ) then
		t1 = t1..CustomLabel[1].." - ";
        else -- Fix for Ticket 8
                CustomLabel[1] = "";
                TitanSpec_CustomLabel = CustomLabel;
	end
	if ( BuildName[1] ~= nil and strtrim(BuildName[1]) ~= "" ) then
		t1 = t1..BuildName[1];
	else
		t1 = t1.."?";
	end

	if ( EquipmentSet[1] ~= nil and strtrim(EquipmentSet[1]) ~= "" ) then
	        t1 = t1.."\n     "..TitanSpec_getColouredText(L["TITAN_SPEC_GEAR_SET"]..": "..EquipmentSet[1], "white");
	end

	if ( GetNumSpecGroups() == 2 ) then
                -- Fix for ticket 7
                if ( CustomLabel[2] == nil or type(CustomLabel[2]) == "boolean" ) then
                        CustomLabel[2] = "";
                        TitanSpec_CustomLabel = CustomLabel;
                end
                if ( BuildName[2] == nil or type(BuildName[2]) == "boolean" ) then
                        BuildName[2] = "";
                        TitanSpec_BuildName = BuildName;
                end
                
		if ( strtrim(CustomLabel[2]) ~= "" ) then
			t2 = t2..CustomLabel[2].." - ";
		end
		if ( strtrim(BuildName[2]) ~= "" ) then
			t2 = t2..BuildName[2];
		else
			t2 = t2.."?";
		end
		if ( strtrim(EquipmentSet[2]) ~= "" ) then
				t2 = t2.."\n     "..TitanSpec_getColouredText(L["TITAN_SPEC_GEAR_SET"]..": "..EquipmentSet[2], "white");
		end
		
		if ( TitanGetVar(TITAN_SPEC_ID, "ShowColours") ) then
			if ( GetActiveSpecGroup() == 1 ) then
				t1 = TitanSpec_getColouredText(t1, "green");
				t2 = TitanSpec_getColouredText(t2, "yellow");
			else
				t1 = TitanSpec_getColouredText(t1, "yellow");
				t2 = TitanSpec_getColouredText(t2, "green");
			end
		end
	else
		t2 = t2..L["TITAN_SPEC_NONE"];
		if ( TitanGetVar(TITAN_SPEC_ID, "ShowColours") ) then
			t1 = TitanSpec_getColouredText(t1, "green");
			t2 = TitanSpec_getColouredText(t2, "red");
		end
	end
	return "\r"..t1.."\r"..t2.."\r".."\r"..L["TITAN_SPEC_HINT"];		
end

function TitanPanelRightClickMenu_PrepareTitanSpecMenu()
	
	local info = {};
	
	if ( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then
		if ( UIDROPDOWNMENU_MENU_VALUE == "TITAN_SPEC_ASSIGN_EQUIPMENT_SET" ) then
                        info = {};
                        info.text = "<"..L["TITAN_SPEC_NONE"]..">";
                        info.func = function()
                            local sets = TitanSpec_EquipmentSet;
                            sets[1] = "";
                            TitanSpec_EquipmentSet = sets;
                            TitanPanelButton_UpdateButton(TITAN_SPEC_ID);
                        end
                        UIDropDownMenu_AddButton(info, 2);
                        
                        local allSets = TitanSpec_EquipmentSet;
		        local numSets = GetNumEquipmentSets();
		        for i=1,numSets do
		                local setName, _, _ = GetEquipmentSetInfo(i);
		                info = {};
                                info.text = setName;
                                if ( allSets[1] == setName ) then
                                        info.text = "|cff00ff00"..info.text..FONT_COLOR_CODE_CLOSE;
                                end
		                info.func = function()
                                    local sets = TitanSpec_EquipmentSet;
                                    sets[1] = setName;
                                    TitanSpec_debug("Selected gear set: "..setName);
                                    TitanSpec_EquipmentSet = sets;
                                    TitanPanelButton_UpdateButton(TITAN_SPEC_ID);
		                end
		                UIDropDownMenu_AddButton(info, 2);
		        end
		elseif ( UIDROPDOWNMENU_MENU_VALUE == "TITAN_SPEC_ASSIGN_EQUIPMENT_SET_2" ) then
                        info = {};
                        info.text = "<"..L["TITAN_SPEC_NONE"]..">";
                        info.func = function()
                            local sets = TitanSpec_EquipmentSet;
                            sets[2] = "";
                            TitanSpec_EquipmentSet =  sets;
                            TitanPanelButton_UpdateButton(TITAN_SPEC_ID);
                        end
                        UIDropDownMenu_AddButton(info, 2);
                        
                        local allSets = TitanSpec_EquipmentSet;
		        local numSets = GetNumEquipmentSets();
		        for i=1,numSets do
		                local setName, _, _ = GetEquipmentSetInfo(i);
		                info = {};
                                info.text = setName;
                                if ( allSets[2] == setName ) then
                                        info.text = "|cff00ff00"..info.text..FONT_COLOR_CODE_CLOSE;
                                end
		                info.func = function()
                                    local sets = TitanSpec_EquipmentSet;
                                    sets[2] = setName;
                                    TitanSpec_debug("Selected gear set: "..setName);
                                    TitanSpec_EquipmentSet = sets;
                                    TitanPanelButton_UpdateButton(TITAN_SPEC_ID);
		                end
		                UIDropDownMenu_AddButton(info, 2);
		        end
        elseif ( UIDROPDOWNMENU_MENU_VALUE == "TITAN_SPEC_EQUIP_EQUIPMENT_SET" ) then
                        info = {};
                        
                        local allSets = TitanSpec_EquipmentSet;
		        local numSets = GetNumEquipmentSets();
		        for i=1,numSets do
		                local setName, _, _ = GetEquipmentSetInfo(i);
		                info = {};
                                info.text = setName;
		                info.func = function()
                                    TitanSpec_debug("Equipping gear set: "..setName);
                                    UseEquipmentSet(setName);
		                end
		                UIDropDownMenu_AddButton(info, 2);
		        end
                end
	else	
            TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_SPEC_ID].menuText);
            TitanPanelRightClickMenu_AddSpacer();
            info.text = L["TITAN_SPEC_RENAME"];
            info.func = function(self)
                    StaticPopupDialogs["TITAN_SPEC_RENAME_DIALOG"] = {
                      text = L["TITAN_SPEC_RENAME_TEXT"],
                      button1 = L["TITAN_SPEC_ACCEPT"],
                      button2 = L["TITAN_SPEC_CANCEL"],
                      OnAccept = function (self, ...)
                            local newName = self.editBox:GetText();
                            local labels = TitanSpec_CustomLabel;
                            labels[GetActiveSpecGroup()] = newName;
                            TitanSpec_CustomLabel = labels;
                            TitanPanelButton_UpdateButton(TITAN_SPEC_ID);
                      end,
                      timeout = 0,
                      whileDead = 1,
                      hideOnEscape = 1,
                      hasEditBox = 1,
                      enterClicksFirstButton = 1
                    };
                    StaticPopup_Show("TITAN_SPEC_RENAME_DIALOG");
            end
            UIDropDownMenu_AddButton(info);
            
            if ( GetNumEquipmentSets() > 0 ) then
                TitanPanelRightClickMenu_AddTitle(L["TITAN_SPEC_ASSIGN_EQUIPMENT_SET"], 2);
                info = {};
                info.text = L["TITAN_SPEC_ASSIGN_EQUIPMENT_SET"];
                info.value = "TITAN_SPEC_ASSIGN_EQUIPMENT_SET";
                info.hasArrow = 1;
                UIDropDownMenu_AddButton(info);
				
				TitanPanelRightClickMenu_AddTitle(L["TITAN_SPEC_ASSIGN_EQUIPMENT_SET_2"], 2);
                info = {};
                info.text = L["TITAN_SPEC_ASSIGN_EQUIPMENT_SET_2"];
                info.value = "TITAN_SPEC_ASSIGN_EQUIPMENT_SET_2";
                info.hasArrow = 1;
                UIDropDownMenu_AddButton(info);
                
                TitanPanelRightClickMenu_AddTitle(L["TITAN_SPEC_EQUIP_EQUIPMENT_SET"], 2);
                info = {};
                info.text = L["TITAN_SPEC_EQUIP_EQUIPMENT_SET"];
                info.value = "TITAN_SPEC_EQUIP_EQUIPMENT_SET";
                info.hasArrow = 1;
                UIDropDownMenu_AddButton(info);
                TitanPanelRightClickMenu_AddSpacer();
            end
            
            TitanPanelRightClickMenu_AddToggleVar(L["TITAN_SPEC_SHOW_COLOURS"], TITAN_SPEC_ID, "ShowColours");
            TitanPanelRightClickMenu_AddToggleVar(L["TITAN_SPEC_SHOW_GEAR_SET"], TITAN_SPEC_ID, "ShowGearSet");
            TitanPanelRightClickMenu_AddToggleLabelText(TITAN_SPEC_ID);
                    
            TitanPanelRightClickMenu_AddSpacer();
            
            info = {};
            info.text = L["TITAN_SPEC_RESET"];	
            info.func = function()
                    TitanSetVar(TITAN_SPEC_ID, "ShowLabelText", 1);
                    TitanSetVar(TITAN_SPEC_ID, "ShowColours", 1);
				   --   Causing issues
                   -- TitanSetVar(TITAN_SPEC_ID, "ShowGearSet", 0);
					TitanSetVar(TITAN_SPEC_ID, "CurrentGearSet", 0);
					TitanSpec_EquipmentSet = {"","",};
					TitanSpec_CustomLabel = {"","",};
					TitanSpec_BuildName = {"","",};
                    TitanSpec_refresh();
					TitanPanelButton_UpdateButton(TITAN_SPEC_ID);
            end
            UIDropDownMenu_AddButton(info);
            TitanPanelRightClickMenu_AddSpacer();
            TitanPanelRightClickMenu_AddCommand(L["TITAN_SPEC_HIDE"], TITAN_SPEC_ID, TITAN_PANEL_MENU_FUNC_HIDE);
        end
end

-- ************************************************************************** --
-- ******************************** User Functions ************************** --
-- ************************************************************************** --

function TitanSpec_debug(msg)
	if ( msg == nil ) then
		msg = "<nil>";
	end
	if IS_DEBUGGING then
		TitanDebug("TitanSpec> "..msg);
	end
end

function TitanSpec_output(msg)
    DEFAULT_CHAT_FRAME:AddMessage(msg, 1, 1, 1);
end


function TitanSpec_getColouredText(msg, colour)
	local colourCode = "";
	
	if ( colour == nil or colour == "default" ) then
		return msg;
	elseif ( colour == "red" ) then
		colourCode = "|cffff0000";
	elseif ( colour == "green" ) then
		colourCode = "|cff00ff00";
	elseif ( colour == "blue" ) then
		colourCode = "|cff0000ff";
	elseif ( colour == "yellow" ) then
		colourCode = "|cffffff00";
	elseif ( colour == "white" ) then
		colourCode = "|cffffffff";
	elseif ( colour == "black" ) then
		colourCode = "|cff000000";
	else
		colourCode = "|cff"..colour;
	end
        
	return colourCode..msg..FONT_COLOR_CODE_CLOSE;
end

